// Copyright (c) 2020 RISC Software GmbH
//
// This file was generated by CPACSGen from CPACS XML Schema (c) German Aerospace Center (DLR/SC).
// Do not edit, all changes are lost when files are re-generated.
//
// Licensed under the Apache License, Version 2.0 (the "License")
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include <cassert>
#include "CCPACSEnginePositions.h"
#include "CPACSEnginePosition.h"
#include "CTiglError.h"
#include "CTiglLogging.h"
#include "CTiglUIDManager.h"
#include "TixiHelper.h"

namespace tigl
{
namespace generated
{
    CPACSEnginePosition::CPACSEnginePosition(CCPACSEnginePositions* parent, CTiglUIDManager* uidMgr)
        : m_uidMgr(uidMgr)
        , m_transformation(reinterpret_cast<CCPACSEnginePosition*>(this), m_uidMgr)
    {
        //assert(parent != NULL);
        m_parent = parent;
    }

    CPACSEnginePosition::~CPACSEnginePosition()
    {
        if (m_uidMgr) m_uidMgr->TryUnregisterObject(m_uID);
        if (m_uidMgr) {
            if (!m_engineUID.empty()) m_uidMgr->TryUnregisterReference(m_engineUID, *this);
            if (!m_parentUID.empty()) m_uidMgr->TryUnregisterReference(m_parentUID, *this);
        }
    }

    const CCPACSEnginePositions* CPACSEnginePosition::GetParent() const
    {
        return m_parent;
    }

    CCPACSEnginePositions* CPACSEnginePosition::GetParent()
    {
        return m_parent;
    }

    const CTiglUIDObject* CPACSEnginePosition::GetNextUIDParent() const
    {
        if (m_parent) {
            return m_parent->GetNextUIDParent();
        }
        return nullptr;
    }

    CTiglUIDObject* CPACSEnginePosition::GetNextUIDParent()
    {
        if (m_parent) {
            return m_parent->GetNextUIDParent();
        }
        return nullptr;
    }

    CTiglUIDManager& CPACSEnginePosition::GetUIDManager()
    {
        return *m_uidMgr;
    }

    const CTiglUIDManager& CPACSEnginePosition::GetUIDManager() const
    {
        return *m_uidMgr;
    }

    void CPACSEnginePosition::ReadCPACS(const TixiDocumentHandle& tixiHandle, const std::string& xpath)
    {
        // read attribute uID
        if (tixi::TixiCheckAttribute(tixiHandle, xpath, "uID")) {
            m_uID = tixi::TixiGetAttribute<std::string>(tixiHandle, xpath, "uID");
            if (m_uID.empty()) {
                LOG(WARNING) << "Required attribute uID is empty at xpath " << xpath;
            }
        }
        else {
            LOG(ERROR) << "Required attribute uID is missing at xpath " << xpath;
        }

        // read attribute symmetry
        if (tixi::TixiCheckAttribute(tixiHandle, xpath, "symmetry")) {
            m_symmetry = stringToTiglSymmetryAxis(tixi::TixiGetAttribute<std::string>(tixiHandle, xpath, "symmetry"));
        }

        // read element name
        if (tixi::TixiCheckElement(tixiHandle, xpath + "/name")) {
            m_name = tixi::TixiGetElement<std::string>(tixiHandle, xpath + "/name");
            if (m_name.empty()) {
                LOG(WARNING) << "Required element name is empty at xpath " << xpath;
            }
        }
        else {
            LOG(ERROR) << "Required element name is missing at xpath " << xpath;
        }

        // read element description
        if (tixi::TixiCheckElement(tixiHandle, xpath + "/description")) {
            m_description = tixi::TixiGetElement<std::string>(tixiHandle, xpath + "/description");
            if (m_description->empty()) {
                LOG(WARNING) << "Optional element description is present but empty at xpath " << xpath;
            }
        }

        // read element engineUID
        if (tixi::TixiCheckElement(tixiHandle, xpath + "/engineUID")) {
            m_engineUID = tixi::TixiGetElement<std::string>(tixiHandle, xpath + "/engineUID");
            if (m_engineUID.empty()) {
                LOG(WARNING) << "Required element engineUID is empty at xpath " << xpath;
            }
            if (m_uidMgr && !m_engineUID.empty()) m_uidMgr->RegisterReference(m_engineUID, *this);
        }
        else {
            LOG(ERROR) << "Required element engineUID is missing at xpath " << xpath;
        }

        // read element parentUID
        if (tixi::TixiCheckElement(tixiHandle, xpath + "/parentUID")) {
            m_parentUID = tixi::TixiGetElement<std::string>(tixiHandle, xpath + "/parentUID");
            if (m_parentUID.empty()) {
                LOG(WARNING) << "Required element parentUID is empty at xpath " << xpath;
            }
            if (m_uidMgr && !m_parentUID.empty()) m_uidMgr->RegisterReference(m_parentUID, *this);
        }
        else {
            LOG(ERROR) << "Required element parentUID is missing at xpath " << xpath;
        }

        // read element transformation
        if (tixi::TixiCheckElement(tixiHandle, xpath + "/transformation")) {
            m_transformation.ReadCPACS(tixiHandle, xpath + "/transformation");
        }
        else {
            LOG(ERROR) << "Required element transformation is missing at xpath " << xpath;
        }

        if (m_uidMgr && !m_uID.empty()) m_uidMgr->RegisterObject(m_uID, *this);
    }

    void CPACSEnginePosition::WriteCPACS(const TixiDocumentHandle& tixiHandle, const std::string& xpath) const
    {
        // write attribute uID
        tixi::TixiSaveAttribute(tixiHandle, xpath, "uID", m_uID);

        // write attribute symmetry
        if (m_symmetry) {
            tixi::TixiSaveAttribute(tixiHandle, xpath, "symmetry", TiglSymmetryAxisToString(*m_symmetry));
        }
        else {
            if (tixi::TixiCheckAttribute(tixiHandle, xpath, "symmetry")) {
                tixi::TixiRemoveAttribute(tixiHandle, xpath, "symmetry");
            }
        }

        // write element name
        tixi::TixiCreateElementIfNotExists(tixiHandle, xpath + "/name");
        tixi::TixiSaveElement(tixiHandle, xpath + "/name", m_name);

        // write element description
        if (m_description) {
            tixi::TixiCreateElementIfNotExists(tixiHandle, xpath + "/description");
            tixi::TixiSaveElement(tixiHandle, xpath + "/description", *m_description);
        }
        else {
            if (tixi::TixiCheckElement(tixiHandle, xpath + "/description")) {
                tixi::TixiRemoveElement(tixiHandle, xpath + "/description");
            }
        }

        // write element engineUID
        tixi::TixiCreateElementIfNotExists(tixiHandle, xpath + "/engineUID");
        tixi::TixiSaveElement(tixiHandle, xpath + "/engineUID", m_engineUID);

        // write element parentUID
        tixi::TixiCreateElementIfNotExists(tixiHandle, xpath + "/parentUID");
        tixi::TixiSaveElement(tixiHandle, xpath + "/parentUID", m_parentUID);

        // write element transformation
        tixi::TixiCreateElementIfNotExists(tixiHandle, xpath + "/transformation");
        m_transformation.WriteCPACS(tixiHandle, xpath + "/transformation");

    }

    const std::string& CPACSEnginePosition::GetUID() const
    {
        return m_uID;
    }

    void CPACSEnginePosition::SetUID(const std::string& value)
    {
        if (m_uidMgr && value != m_uID) {
            if (m_uID.empty()) {
                m_uidMgr->RegisterObject(value, *this);
            }
            else {
                m_uidMgr->UpdateObjectUID(m_uID, value);
            }
        }
        m_uID = value;
    }

    const boost::optional<TiglSymmetryAxis>& CPACSEnginePosition::GetSymmetry() const
    {
        return m_symmetry;
    }

    void CPACSEnginePosition::SetSymmetry(const boost::optional<TiglSymmetryAxis>& value)
    {
        m_symmetry = value;
    }

    const std::string& CPACSEnginePosition::GetName() const
    {
        return m_name;
    }

    void CPACSEnginePosition::SetName(const std::string& value)
    {
        m_name = value;
    }

    const boost::optional<std::string>& CPACSEnginePosition::GetDescription() const
    {
        return m_description;
    }

    void CPACSEnginePosition::SetDescription(const boost::optional<std::string>& value)
    {
        m_description = value;
    }

    const std::string& CPACSEnginePosition::GetEngineUID() const
    {
        return m_engineUID;
    }

    void CPACSEnginePosition::SetEngineUID(const std::string& value)
    {
        if (m_uidMgr) {
            if (!m_engineUID.empty()) m_uidMgr->TryUnregisterReference(m_engineUID, *this);
            if (!value.empty()) m_uidMgr->RegisterReference(value, *this);
        }
        m_engineUID = value;
    }

    const std::string& CPACSEnginePosition::GetParentUID() const
    {
        return m_parentUID;
    }

    void CPACSEnginePosition::SetParentUID(const std::string& value)
    {
        if (m_uidMgr) {
            if (!m_parentUID.empty()) m_uidMgr->TryUnregisterReference(m_parentUID, *this);
            if (!value.empty()) m_uidMgr->RegisterReference(value, *this);
        }
        m_parentUID = value;
    }

    const CCPACSTransformation& CPACSEnginePosition::GetTransformation() const
    {
        return m_transformation;
    }

    CCPACSTransformation& CPACSEnginePosition::GetTransformation()
    {
        return m_transformation;
    }

    const CTiglUIDObject* CPACSEnginePosition::GetNextUIDObject() const
    {
        return this;
    }

    void CPACSEnginePosition::NotifyUIDChange(const std::string& oldUid, const std::string& newUid)
    {
        if (m_engineUID == oldUid) {
            m_engineUID = newUid;
        }
        if (m_parentUID == oldUid) {
            m_parentUID = newUid;
        }
    }

} // namespace generated
} // namespace tigl
